#!/usr/bin/ruby

# Copyright (C) 2014 Open Source Robotics Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# We use 'dl' for Ruby <= 1.9.x and 'fiddle' for Ruby >= 2.0.x
if RUBY_VERSION.split('.')[0] < '2'
  require 'dl'
  require 'dl/import'
  include DL
else
  require 'fiddle'
  require 'fiddle/import'
  include Fiddle
end

require 'optparse'

# Constants.
LIBRARY_NAME = 'libignition-transport2.dylib'
LIBRARY_VERSION = '2.1.0'
COMMON_OPTIONS =
               "  -h [ --help ]              Print this help message.\n"\
               "  --force-version <VERSION>  Use a specific library version.\n"\
               '  --versions                 Show the available versions.'
COMMANDS = {  'topic' =>
                       "Print information about topics.\n\n" +
                       "  ign topic [options]\n\n" +
                       "Options:\n\n" +
                       "  -i [ --info ]              Get info about a topic.\n"+
                       "                             Requires the -t option.\n"+
                       "  -l [ --list ]              List all topics.\n"       +
                       "  -t [ --topic ] arg         Name of a topic.\n"       +
                       "                             Required with -i, -p.\n"  +
                       "  -m [ --msgtype ] arg       Type of message to "      +
                       "publish.\n"                                            +
                       "                             Arg is a message type.\n" +
                       "                             Required with -p.\n"      +
                       "  -p [ --pub ] arg           Publish a message.\n"     +
                       "                             Arg is the message data." +
                       " The format expected is\n                            " +
                       " the same used by Protobuf DebugString(). E.g.:\n\n"   +
                       "                               ign topic -t /foo -m "  +
                       "  ignition.msgs.StringMsg\n                          " +
                       "     -p \'data:\"Custom data\"\'\n\n"                +
                       "                             Requires -t and -m.\n"    +
                       COMMON_OPTIONS,
              'service' =>
                       "Print information about services.\n\n"\
                       "  ign service [options]\n\n"\
                       "Options:\n\n"\
                       "  -i [ --info ]              Get info about a "        +
                       "service.\n"                                            +
                       "  -s [ --service ] arg       Name of a service.\n"     +
                       "                             Required with -i.\n"      +
                       "                             'arg' is a service "      +
                       "name.\n"                                               +
                       "  -l [ --list ]              List all services.\n"     +
                       COMMON_OPTIONS
            }

#
# Class for the Ignition transport command line tools.
#
class Cmd
  #
  # Return a structure describing the options.
  #
  def parse(args)
    options = {}

    usage = COMMANDS[args[0]]

    # Read the command line arguments.
    opt_parser = OptionParser.new do |opts|
      opts.banner = usage

      opts.on('-h', '--help", "Print this help message') do
        puts usage
        exit(0)
      end
      opts.on('-l', '--list', 'Print information about topics') do |l|
        options['list'] = l
      end

      opts.on('-t topic', '--topic', String,
              'Topic name') do |t|
        options['topic'] = t
      end

      opts.on('-s service', '--service', String,
              'Service name') do |t|
        options['service'] = t
      end

      opts.on('-m type', '--msgtype', String,
              'Message type') do |t|
        options['msgtype'] = t
      end

      opts.on('-p data', '--pub', String,
              'Publish a message') do |t|
        options['pub'] = t
      end

      opts.on('-i', '--info', String,
              'Print information about a topic') do |t|
        options['info'] = t
      end
    end
    begin
      opt_parser.parse!(args)
    rescue
      puts usage
      exit(-1)
    end

    # Check that there is at least one command and there is a plugin that knows
    # how to handle it.
    if ARGV.empty? || !COMMANDS.key?(ARGV[0]) ||
       options.empty?
      puts usage
      exit(-1)
    end

    options['command'] = ARGV[0]

    options
  end  # parse()

  def execute(args)
    options = parse(args)

    # puts 'Parsed:'
    # puts options

    # Read the plugin that handles the command.
    plugin = LIBRARY_NAME
    conf_version = LIBRARY_VERSION

    begin
      Importer.dlload plugin
    rescue DLError
      puts "Library error: [#{plugin}] not found."
      exit(-1)
    end

    # Read the library version.
    Importer.extern 'char* ignitionVersion()'
    begin
      plugin_version = Importer.ignitionVersion.to_s
    rescue DLError
      puts "Library error: Problem running 'ignitionVersion()' from #{plugin}."
      exit(-1)
    end

    # Sanity check: Verify that the version of the yaml file matches the version
    # of the library that we are using.
    unless plugin_version.eql? conf_version
      puts "Error: Version mismatch. Your configuration file version is
            [#{conf_version}] but #{plugin} version is [#{plugin_version}]."
      exit(-1)
    end

    begin
      case options['command']
      when 'topic'
        if options.key?('list')
          Importer.extern 'void cmdTopicList()'
          Importer.cmdTopicList
        elsif options.key?('info')
          Importer.extern 'void cmdTopicInfo(const char *)'
          Importer.cmdTopicInfo(options['topic'])
        elsif options.key?('pub')
          Importer.extern 'void cmdTopicPub(const char *,const char *, const char *)'
          Importer.cmdTopicPub(options['topic'], options['msgtype'],
                               options['pub'])
        else
          puts 'Command error: I do not have an implementation '\
               'for this command.'
        end
      when 'service'
        if options.key?('list')
          Importer.extern 'void cmdServiceList()'
          Importer.cmdServiceList
        elsif options.key?('info')
          Importer.extern 'void cmdServiceInfo(const char *)'
          Importer.cmdServiceInfo(options['service'])
        else
          puts 'Command error: I do not have an implementation '\
               'for this command.'
        end
      else
        puts 'Command error: I do not have an implementation for '\
             "command [ign #{options['command']}]."
      end
    rescue
      puts "Library error: Problem running [#{options['command']}]() "\
           "from #{plugin}."
    end
  end
end
